import calendar


class Ticket(object):
    """An object that represents a ticket. The important part of this
    object is that it can compare itself to other ticket objects in
    order to sort them by a complex algorithm.
    """

    def __init__(self, id, summary, milestone, priority, flag, time_remaining):
        self.id = id
        self.summary = summary
        self.milestone = milestone
        self.priority = priority
        self.flag = flag
        self.time_remaining = time_remaining

    def __repr__(self):
        return "<Ticket #%s>" % self.id

    def __cmp__(self, other):
        """This is effin' nutso. Bear with me here.

       Sort by the following:

            - Milestone
              - "Week of ..." goes first
            - Flag
              - Non days go first
              - Then by order of weekday (nn-day, 01-Monday)
            - Priority
              - Trac defined order of priorities
        """
        milestone = self.milestone.lower()
        other_milestone = other.milestone.lower()
        priority = self.priority.lower()
        other_priority = other.priority.lower()
        flag = self.flag
        other_flag = other.flag
        day_name = flag[3:]
        other_day_name = flag[3:]
        weekdays = [day.lower() for day in calendar.day_name]
        wix = weekdays.index
        # milestone and flag are the same, so we compare with the priority
        if milestone == other_milestone and flag == other_flag:
            # XXX: compare priorities based on trac order
            return cmp(priority, other_priority)
        # milestones are the same, but flags differ
        if milestone == other_milestone and flag != other_flag:
            # priorities that are not day names come first
            if day_name in weekdays and other_day_name not in weekdays:
                return 1
            # if we are dealing with two days, compare their index in
            # the weekday list
            elif day_name in weekdays and other_day_name in weekdays:
                return cmp(wix(day_name), wix(other_day_name))
            # otherwise just compare the original strings
            else:
                return cmp(flag, other_flag)
        # milestones are not the same
        if milestone != other_milestone:
            m_weekof = milestone.startswith("week of")
            o_weekof = other_milestone.startswith("week of")
            # both start with 'week of' compare them directly
            if m_weekof and o_weekof:
                return cmp(milestone, other_milestone)
            # the other item is something other than 'week of'
            elif m_weekof and not o_weekof:
                return -1
            # this item is something other than 'week of'
            elif not m_weekof and o_weekof:
                return 1
            # neither are 'week of' compare them directly
            else:
                return cmp(milestone, other_milestone)
